@@ -26,7 +26,6 @@
 #include <apt-pkg/aptconfiguration.h>
 #include <apt-pkg/configuration.h>
 #include <apt-pkg/macros.h>
-#include <apt-pkg/pkgsystem.h>
 
 #include <ctype.h>
 #include <stdarg.h>
@@ -67,12 +66,6 @@
 #ifdef HAVE_LZ4
 	#include <lz4frame.h>
 #endif
-#ifdef HAVE_SYSTEMD
-#include <systemd/sd-bus.h>
-#endif
-#ifdef HAVE_ZSTD
-#include <zstd.h>
-#endif
 #include <endian.h>
 #include <stdint.h>
 
@@ -104,8 +97,6 @@ bool RunScripts(const char *Cnf)
    // This is the child
    if (Child == 0)
    {
-      if (_system != nullptr && _system->IsLocked() == true && (stringcasecmp(Cnf, "dpkg::post-invoke") == 0 || stringcasecmp(Cnf, "dpkg::pre-invoke") == 0))
-	 setenv("DPKG_FRONTEND_LOCKED", "true", 1);
       if (_config->FindDir("DPkg::Chroot-Directory","/") != "/") 
       {
          std::cerr << "Chrooting into " 
@@ -703,6 +694,25 @@ string flAbsPath(string File)
    return AbsPath;
 }
 									/*}}}*/
+std::string flNormalize(std::string file)				/*{{{*/
+{
+   if (file.empty())
+      return file;
+   // do some normalisation by removing // and /./ from the path
+   size_t found = string::npos;
+   while ((found = file.find("/./")) != string::npos)
+      file.replace(found, 3, "/");
+   while ((found = file.find("//")) != string::npos)
+      file.replace(found, 2, "/");
+
+   if (APT::String::Startswith(file, "/dev/null"))
+   {
+      file.erase(strlen("/dev/null"));
+      return file;
+   }
+   return file;
+}
+									/*}}}*/
 // SetCloseExec - Set the close on exec flag				/*{{{*/
 // ---------------------------------------------------------------------
 /* */
@@ -1637,11 +1647,14 @@ public:
 
       if (cctx != nullptr)
       {
-	 res = LZ4F_compressEnd(cctx, lz4_buffer.buffer, lz4_buffer.buffersize_max, nullptr);
-	 if (LZ4F_isError(res) || backend.Write(lz4_buffer.buffer, res) == false)
-	    return false;
-	 if (!backend.Flush())
-	    return false;
+	 if (filefd->Failed() == false)
+	 {
+	    res = LZ4F_compressEnd(cctx, lz4_buffer.buffer, lz4_buffer.buffersize_max, nullptr);
+	    if (LZ4F_isError(res) || backend.Write(lz4_buffer.buffer, res) == false)
+	       return false;
+	    if (!backend.Flush())
+	       return false;
+	 }
 	 if (!backend.Close())
 	    return false;
 
@@ -1668,234 +1681,23 @@ public:
       InternalClose("");
    }
 #endif
-};
-									/*}}}*/
-
-class APT_HIDDEN ZstdFileFdPrivate : public FileFdPrivate
-{ /*{{{*/
-#ifdef HAVE_ZSTD
-   ZSTD_DStream *dctx;
-   ZSTD_CStream *cctx;
-   size_t res;
-   FileFd backend;
-   simple_buffer zstd_buffer;
-   // Count of bytes that the decompressor expects to read next, or buffer size.
-   size_t next_to_load = APT_BUFFER_SIZE;
-
-   public:
-   virtual bool InternalOpen(int const iFd, unsigned int const Mode) APT_OVERRIDE
-   {
-      if ((Mode & FileFd::ReadWrite) == FileFd::ReadWrite)
-	 return _error->Error("zstd only supports write or read mode");
-
-      if ((Mode & FileFd::WriteOnly) == FileFd::WriteOnly)
-      {
-	 cctx = ZSTD_createCStream();
-	 res = ZSTD_initCStream(cctx, findLevel(compressor.CompressArgs));
-	 zstd_buffer.reset(APT_BUFFER_SIZE);
-      }
-      else
-      {
-	 dctx = ZSTD_createDStream();
-	 res = ZSTD_initDStream(dctx);
-	 zstd_buffer.reset(APT_BUFFER_SIZE);
-      }
-
-      filefd->Flags |= FileFd::Compressed;
-
-      if (ZSTD_isError(res))
-	 return false;
-
-      unsigned int flags = (Mode & (FileFd::WriteOnly | FileFd::ReadOnly));
-      if (backend.OpenDescriptor(iFd, flags, FileFd::None, true) == false)
-	 return false;
-
-      return true;
-   }
-   virtual ssize_t InternalUnbufferedRead(void *const To, unsigned long long const Size) APT_OVERRIDE
-   {
-      /* Keep reading as long as the compressor still wants to read */
-      while (true)
-      {
-	 // Fill compressed buffer;
-	 if (zstd_buffer.empty())
-	 {
-	    unsigned long long read;
-	    /* Reset - if LZ4 decompressor wants to read more, allocate more */
-	    zstd_buffer.reset(next_to_load);
-	    if (backend.Read(zstd_buffer.getend(), zstd_buffer.free(), &read) == false)
-	       return -1;
-	    zstd_buffer.bufferend += read;
-
-	    if (read == 0)
-	    {
-	       /* Expected EOF */
-	       if (next_to_load == 0)
-		  return 0;
-
-	       res = -1;
-	       return filefd->FileFdError("ZSTD: %s %s",
-					  filefd->FileName.c_str(),
-					  _("Unexpected end of file")),
-		      -1;
-	    }
-	 }
-	 // Drain compressed buffer as far as possible.
-	 ZSTD_inBuffer in = {
-	    .src = zstd_buffer.get(),
-	    .size = zstd_buffer.size(),
-	    .pos = 0,
-	 };
-	 ZSTD_outBuffer out = {
-	    .dst = To,
-	    .size = Size,
-	    .pos = 0,
-	 };
-
-	 next_to_load = res = ZSTD_decompressStream(dctx, &out, &in);
-
-	 if (res == 0)
-	 {
-	    res = ZSTD_initDStream(dctx);
-	 }
-
-	 if (ZSTD_isError(res))
-	    return -1;
-
-	 zstd_buffer.bufferstart += in.pos;
-
-	 if (out.pos != 0)
-	    return out.pos;
-      }
-
-      return 0;
-   }
-   virtual bool InternalReadError() APT_OVERRIDE
-   {
-      char const *const errmsg = ZSTD_getErrorName(res);
-
-      return filefd->FileFdError("ZSTD: %s %s (%zu: %s)", filefd->FileName.c_str(), _("Read error"), res, errmsg);
-   }
-   virtual ssize_t InternalWrite(void const *const From, unsigned long long const Size) APT_OVERRIDE
-   {
-      // Drain compressed buffer as far as possible.
-      ZSTD_outBuffer out = {
-	 .dst = zstd_buffer.buffer,
-	 .size = zstd_buffer.buffersize_max,
-	 .pos = 0,
-      };
-      ZSTD_inBuffer in = {
-	 .src = From,
-	 .size = Size,
-	 .pos = 0,
-      };
-
-      res = ZSTD_compressStream(cctx, &out, &in);
-
-      if (ZSTD_isError(res) || backend.Write(zstd_buffer.buffer, out.pos) == false)
-	 return -1;
-
-      return in.pos;
-   }
-
-   virtual bool InternalWriteError() APT_OVERRIDE
-   {
-      char const *const errmsg = ZSTD_getErrorName(res);
-
-      return filefd->FileFdError("ZSTD: %s %s (%zu: %s)", filefd->FileName.c_str(), _("Write error"), res, errmsg);
-   }
-   virtual bool InternalStream() const APT_OVERRIDE { return true; }
-
-   virtual bool InternalFlush() APT_OVERRIDE
-   {
-      return backend.Flush();
-   }
-
-   virtual bool InternalClose(std::string const &) APT_OVERRIDE
-   {
-      /* Reset variables */
-      res = 0;
-      next_to_load = APT_BUFFER_SIZE;
-
-      if (cctx != nullptr)
-      {
-	 if (filefd->Failed() == false)
-	 {
-	    do
-	    {
-	       ZSTD_outBuffer out = {
-		  .dst = zstd_buffer.buffer,
-		  .size = zstd_buffer.buffersize_max,
-		  .pos = 0,
-	       };
-	       res = ZSTD_endStream(cctx, &out);
-	       if (ZSTD_isError(res) || backend.Write(zstd_buffer.buffer, out.pos) == false)
-		  return false;
-	    } while (res > 0);
-
-	    if (!backend.Flush())
-	       return false;
-	 }
-	 if (!backend.Close())
-	    return false;
-
-	 res = ZSTD_freeCStream(cctx);
-	 cctx = nullptr;
-      }
-
-      if (dctx != nullptr)
-      {
-	 res = ZSTD_freeDStream(dctx);
-	 dctx = nullptr;
-      }
-      if (backend.IsOpen())
-      {
-	 backend.Close();
-	 filefd->iFd = -1;
-      }
-
-      return ZSTD_isError(res) == false;
-   }
-
-   static uint32_t findLevel(std::vector<std::string> const &Args)
-   {
-      for (auto a = Args.rbegin(); a != Args.rend(); ++a)
-      {
-	 if (a->size() >= 2 && (*a)[0] == '-' && (*a)[1] != '-')
-	 {
-	    auto const level = a->substr(1);
-	    auto const notANumber = level.find_first_not_of("0123456789");
-	    if (notANumber != std::string::npos)
-	       continue;
-
-	    return (uint32_t)stoi(level);
-	 }
-      }
-      return 19;
-   }
-
-   explicit ZstdFileFdPrivate(FileFd *const filefd) : FileFdPrivate(filefd), dctx(nullptr), cctx(nullptr) {}
-   virtual ~ZstdFileFdPrivate()
-   {
-      InternalClose("");
-   }
-#endif
 };
 									/*}}}*/
 class APT_HIDDEN LzmaFileFdPrivate: public FileFdPrivate {				/*{{{*/
 #ifdef HAVE_LZMA
    struct LZMAFILE {
       FILE* file;
+      FileFd * const filefd;
       uint8_t buffer[4096];
       lzma_stream stream;
       lzma_ret err;
       bool eof;
       bool compressing;
 
-      LZMAFILE() : file(nullptr), eof(false), compressing(false) { buffer[0] = '\0'; }
+      LZMAFILE(FileFd * const fd) : file(nullptr), filefd(fd), eof(false), compressing(false) { buffer[0] = '\0'; }
       ~LZMAFILE()
       {
-	 if (compressing == true)
+	 if (compressing == true && filefd->Failed() == false)
 	 {
 	    size_t constexpr buffersize = sizeof(buffer)/sizeof(buffer[0]);
 	    while(true)
@@ -1956,7 +1758,7 @@ public:
 	 return filefd->FileFdError("ReadWrite mode is not supported for lzma/xz files %s", filefd->FileName.c_str());
 
       if (lzma == nullptr)
-	 lzma = new LzmaFileFdPrivate::LZMAFILE;
+	 lzma = new LzmaFileFdPrivate::LZMAFILE(filefd);
       if ((Mode & FileFd::WriteOnly) == FileFd::WriteOnly)
 	 lzma->file = fdopen(iFd, "w");
       else
@@ -2101,6 +1903,9 @@ public:
 
       if ((Mode & FileFd::ReadWrite) == FileFd::ReadWrite)
 	 return filefd->FileFdError("ReadWrite mode is not supported for file %s", filefd->FileName.c_str());
+      if (compressor.Binary == "false")
+	 return filefd->FileFdError("libapt has inbuilt support for the %s compression,"
+	       " but was forced to ignore it in favor of an external binary – which isn't installed.", compressor.Name.c_str());
 
       bool const Comp = (Mode & FileFd::WriteOnly) == FileFd::WriteOnly;
       if (Comp == false && filefd->iFd != -1)
@@ -2374,7 +2179,6 @@ bool FileFd::Open(string FileName,unsigned int const Mode,CompressMode Compress,
       case Lzma: name = "lzma"; break;
       case Xz: name = "xz"; break;
       case Lz4: name = "lz4"; break;
-      case Zstd: name = "zstd"; break;
       case Auto:
       case Extension:
 	 // Unreachable
@@ -2492,7 +2296,6 @@ bool FileFd::OpenDescriptor(int Fd, unsigned int const Mode, CompressMode Compre
    case Lzma: name = "lzma"; break;
    case Xz: name = "xz"; break;
    case Lz4: name = "lz4"; break;
-   case Zstd: name = "zstd"; break;
    case Auto:
    case Extension:
       if (AutoClose == true && Fd != -1)
@@ -2557,9 +2360,6 @@ bool FileFd::OpenInternDescriptor(unsigned int const Mode, APT::Configuration::C
 #ifdef HAVE_LZ4
       APT_COMPRESS_INIT("lz4", Lz4FileFdPrivate);
 #endif
-#ifdef HAVE_ZSTD
-      APT_COMPRESS_INIT("zstd", ZstdFileFdPrivate);
-#endif
 #undef APT_COMPRESS_INIT
       else if (compressor.Name == "." || compressor.Binary.empty() == true)
 	 d = new DirectFileFdPrivate(this);
@@ -2602,7 +2402,7 @@ FileFd::~FileFd()
    gracefully. */
 bool FileFd::Read(void *To,unsigned long long Size,unsigned long long *Actual)
 {
-   if (d == nullptr)
+   if (d == nullptr || Failed())
       return false;
    ssize_t Res = 1;
    errno = 0;
@@ -2644,6 +2444,37 @@ bool FileFd::Read(void *To,unsigned long long Size,unsigned long long *Actual)
    }
 
    return FileFdError(_("read, still have %llu to read but none left"), Size);
+}
+bool FileFd::Read(int const Fd, void *To, unsigned long long Size, unsigned long long * const Actual)
+{
+   ssize_t Res = 1;
+   errno = 0;
+   if (Actual != nullptr)
+      *Actual = 0;
+   *static_cast<char *>(To) = '\0';
+   while (Res > 0 && Size > 0)
+   {
+      Res = read(Fd, To, Size);
+      if (Res < 0)
+      {
+	 if (errno == EINTR)
+	 {
+	    Res = 1;
+	    errno = 0;
+	    continue;
+	 }
+	 return _error->Errno("read", _("Read error"));
+      }
+      To = static_cast<char *>(To) + Res;
+      Size -= Res;
+      if (Actual != 0)
+	 *Actual += Res;
+   }
+   if (Size == 0)
+      return true;
+   if (Actual != nullptr)
+      return true;
+   return _error->Error(_("read, still have %llu to read but none left"), Size);
 }
 									/*}}}*/
 // FileFd::ReadLine - Read a complete line from the file		/*{{{*/
@@ -2653,7 +2484,7 @@ bool FileFd::Read(void *To,unsigned long long Size,unsigned long long *Actual)
 char* FileFd::ReadLine(char *To, unsigned long long const Size)
 {
    *To = '\0';
-   if (d == nullptr)
+   if (d == nullptr || Failed())
       return nullptr;
    return d->InternalReadLine(To, Size);
 }
@@ -2661,6 +2492,8 @@ char* FileFd::ReadLine(char *To, unsigned long long const Size)
 // FileFd::Flush - Flush the file  					/*{{{*/
 bool FileFd::Flush()
 {
+   if (Failed())
+      return false;
    if (d == nullptr)
       return true;
 
@@ -2670,7 +2503,7 @@ bool FileFd::Flush()
 // FileFd::Write - Write to the file					/*{{{*/
 bool FileFd::Write(const void *From,unsigned long long Size)
 {
-   if (d == nullptr)
+   if (d == nullptr || Failed())
       return false;
    ssize_t Res = 1;
    errno = 0;
@@ -2726,7 +2559,7 @@ bool FileFd::Write(int Fd, const void *From, unsigned long long Size)
 // FileFd::Seek - Seek in the file					/*{{{*/
 bool FileFd::Seek(unsigned long long To)
 {
-   if (d == nullptr)
+   if (d == nullptr || Failed())
       return false;
    Flags &= ~HitEof;
    return d->InternalSeek(To);
@@ -2735,7 +2568,7 @@ bool FileFd::Seek(unsigned long long To)
 // FileFd::Skip - Skip over data in the file				/*{{{*/
 bool FileFd::Skip(unsigned long long Over)
 {
-   if (d == nullptr)
+   if (d == nullptr || Failed())
       return false;
    return d->InternalSkip(Over);
 }
@@ -2743,7 +2576,7 @@ bool FileFd::Skip(unsigned long long Over)
 // FileFd::Truncate - Truncate the file					/*{{{*/
 bool FileFd::Truncate(unsigned long long To)
 {
-   if (d == nullptr)
+   if (d == nullptr || Failed())
       return false;
    // truncating /dev/null is always successful - as we get an error otherwise
    if (To == 0 && FileName == "/dev/null")
@@ -2756,7 +2589,7 @@ bool FileFd::Truncate(unsigned long long To)
 /* */
 unsigned long long FileFd::Tell()
 {
-   if (d == nullptr)
+   if (d == nullptr || Failed())
       return false;
    off_t const Res = d->InternalTell();
    if (Res == (off_t)-1)
@@ -2819,7 +2652,7 @@ time_t FileFd::ModificationTime()
 unsigned long long FileFd::Size()
 {
    if (d == nullptr)
-      return false;
+      return 0;
    return d->InternalSize();
 }
 									/*}}}*/
@@ -2828,7 +2661,7 @@ unsigned long long FileFd::Size()
 /* */
 bool FileFd::Close()
 {
-   if (Flush() == false)
+   if (Failed() == false && Flush() == false)
       return false;
    if (iFd == -1)
       return true;
@@ -2945,9 +2778,9 @@ std::vector<std::string> Glob(std::string const &pattern, int flags)
    return result;
 }
 									/*}}}*/
-static std::string APT_NONNULL(1) GetTempDirEnv(char const * const env)	/*{{{*/
+std::string GetTempDir()						/*{{{*/
 {
-   const char *tmpdir = getenv(env);
+   const char *tmpdir = getenv("TMPDIR");
 
 #ifdef P_tmpdir
    if (!tmpdir)
@@ -2959,15 +2792,10 @@ static std::string APT_NONNULL(1) GetTempDirEnv(char const * const env)	/*{{{*/
 	 stat(tmpdir, &st) != 0 || (st.st_mode & S_IFDIR) == 0) // exists and is directory
       tmpdir = "/tmp";
    else if (geteuid() != 0 && // root can do everything anyway
-	 faccessat(AT_FDCWD, tmpdir, R_OK | W_OK | X_OK, AT_EACCESS) != 0) // current user has rwx access to directory
+	 faccessat(-1, tmpdir, R_OK | W_OK | X_OK, AT_EACCESS | AT_SYMLINK_NOFOLLOW) != 0) // current user has rwx access to directory
       tmpdir = "/tmp";
 
    return string(tmpdir);
-}
-									/*}}}*/
-std::string GetTempDir()						/*{{{*/
-{
-   return GetTempDirEnv("TMPDIR");
 }
 std::string GetTempDir(std::string const &User)
 {
@@ -3223,77 +3051,6 @@ bool DropPrivileges()							/*{{{*/
 	 return _error->Error("Could restore a uid to root, privilege dropping did not work");
    }
 
-   if (_config->FindB("APT::Sandbox::ResetEnvironment", true))
-   {
-      setenv("HOME", pw->pw_dir, 1);
-      setenv("USER", pw->pw_name, 1);
-      setenv("USERNAME", pw->pw_name, 1);
-      setenv("LOGNAME", pw->pw_name, 1);
-      auto const shell = flNotDir(pw->pw_shell);
-      if (shell == "false" || shell == "nologin")
-	 setenv("SHELL", "/bin/sh", 1);
-      else
-	 setenv("SHELL", pw->pw_shell, 1);
-      auto const apt_setenv_tmp = [](char const * const env) {
-	 auto const tmpdir = getenv(env);
-	 if (tmpdir != nullptr)
-	 {
-	    auto const ourtmpdir = GetTempDirEnv(env);
-	    if (ourtmpdir != tmpdir)
-	       setenv(env, ourtmpdir.c_str(), 1);
-	 }
-      };
-      apt_setenv_tmp("TMPDIR");
-      apt_setenv_tmp("TEMPDIR");
-      apt_setenv_tmp("TMP");
-      apt_setenv_tmp("TEMP");
-   }
-
    return true;
 }
 									/*}}}*/
-int Inhibit(const char *what, const char *who, const char *why, const char *mode) /*{{{*/
-{
-#ifdef HAVE_SYSTEMD
-   sd_bus_error error = SD_BUS_ERROR_NULL;
-   sd_bus_message *m = NULL;
-   sd_bus *bus = NULL;
-   int fd;
-   int r;
-
-   r = sd_bus_open_system(&bus);
-   if (r < 0)
-      goto out;
-
-   r = sd_bus_call_method(bus,
-			  "org.freedesktop.login1",
-			  "/org/freedesktop/login1",
-			  "org.freedesktop.login1.Manager",
-			  "Inhibit",
-			  &error,
-			  &m,
-			  "ssss",
-			  what,
-			  who,
-			  why,
-			  mode);
-   if (r < 0)
-      goto out;
-
-   r = sd_bus_message_read(m, "h", &fd);
-   if (r < 0)
-      goto out;
-
-   // We received a file descriptor, return it - systemd will close the read fd
-   // on free, so let's duplicate it here.
-   r = dup(fd);
-out:
-   sd_bus_error_free(&error);
-   sd_bus_message_unref(m);
-   sd_bus_unref(bus);
-   return r;
-#else
-   return -ENOTSUP;
-#endif
-}
-									/*}}}*/
